==> bee_hotel-2.13.x-dev.tar.gz: bee_hotel/src/Form/BookThisUnitForm.php <==
<?php

namespace Drupal\bee_hotel\Form;

use Drupal\bee_hotel\Util\BeeHotelUnit;
use Drupal\bee_hotel\Util\Dates;
use Drupal\commerce_product\Entity\Product;
use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\node\NodeInterface;
use Drupal\node\Entity\Node;

define('FIELD_PRODUCT', 'field_product');

/**
 * Reservation form for a unit.
 *
 * @package Drupal\bee_hotel
 *   Defines the reservation form for a unit
 */
class BookThisUnitForm extends FormBase {

  /**
   * Constructs the object.
   */
  public function __construct() {
    $this->beehotelunit = new BeeHotelUnit($data = []);
    $this->beehoteldates = new Dates();
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'bee_hotel_book_this_unit_form';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state, NodeInterface $node = NULL) {

    $config = $this->config('beehotel.settings');

    if ($config->get("beehotel.off_value")) {
      \Drupal::messenger()->addWarning($config->get("beehotel.off_text"));
      return;
    }

    $data = [];

    $data['bee_settings'] = \Drupal::config('node.type.' . $node->bundle())->get('bee');

    $data['default_values']['dates'] = date("j M Y", strtotime("+1 day")) . " - " . date("j M Y", strtotime("+3 day"));

    $data['request_values']['row'] = \Drupal::request()->get('v');

    // Pre-fill for Litepicker.
    if (!empty($data['request_values']['row'])) {
      $data['request_values']['pieces'] = explode("-", $data['request_values']['row']);
      $this->beehoteldates->fromRequestToLitepicker($data);
    }

    $form['node'] = [
      '#type' => 'hidden',
      '#value' => $node->id(),
    ];

    $form['title'] = [
      '#type' => 'html_tag',
      '#tag' => 'h2',
      '#value' => $this
        ->t('Book @t now', ['@t' => $node->getTitle()]),
    ];

    $form['dates'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Check in > Check out'),
      '#default_value' => $data['default_values']['dates'],
      '#required' => TRUE,
    ];

    $product_field = $node->get(FIELD_PRODUCT)->getValue();
    $pid = $product_field[0]['target_id'];

    $data['max_occupancy'] = $this->beehotelunit->maxOccupancy($node);
    $options = [];
    for ($o = 1; $o <= $data['max_occupancy']; $o++) {
      $options[$o] = $o;
    }

    $form['guests'] = [
      '#default_value' => trim($data['request_values']['pieces'][6]),
      '#type' => 'select',
      '#title' => $this->t('Guests'),
      '#options' => $options,
    ];

    $form['pid'] = [
      '#type' => 'hidden',
      '#value' => $pid,
    ];

    $form['submit'] = [
      '#type' => 'submit',
      '#value' => t('Check availability'),
    ];

    $form['#attached']['library'][] = 'bee_hotel/book-this-unit';
    $form['#weight'] = -200;

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {

    $data = [];
    $data['values'] = $form_state->getValues();
    $node = Node::load($data['values']['node']);
    $bee_settings = \Drupal::config('node.type.' . $node->bundle())->get('bee');
    $data = $this->beehotelunit->getAvailableUnits($data);

    if (empty($data['available_units'])) {
      $form_state->setError($form, t('Sorry, no Unit available now'));
    }

    // Payment validation.
    if ($bee_settings['payment'] < 1) {
      $form_state->setErrorByName('checkin', $this->t('No payment available for this Content type. Please check "Enable payment for bookings"'));
    }

    if ($node->get("field_accept_reservations")->value != 1) {
      $form_state->setErrorByName('checkin', $this->t('Sorry, no Unit available now...'));
    }

  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {

    $data = [];
    $data['values'] = $form_state->getValues();
    $node = Node::load($data['values']['node']);
    $bee_settings = \Drupal::config('node.type.' . $node->bundle())->get('bee');
    $data = $this->beehotelunit->getAvailableUnits($data);

    $booking = bat_booking_create([
      'type' => 'bee',
      'label' => $node->label(),
    ]);

    $booking->set('booking_start_date', $data['norm']['dates_from_search_form']['checkin']['Y-m-d-H-i-s']);
    $booking->set('booking_end_date', $data['norm']['dates_from_search_form']['checkout']['Y-m-d-H-i-s']);
    $booking->save();

    $product = $node->get(FIELD_PRODUCT)->entity;

    $stores = $product->getStores();
    $store = reset($stores);

    /*Load Product*/
    $product = Product::load((int) $data['values']['pid']);

    /*Load Product Variations*/
    $entity_manager = \Drupal::EntityTypeManager();
    $variations = $product->getVariationIds();

    $options = [];

    // This works fine for single slot units.
    foreach ($variations as $v) {
      $product_variation = $entity_manager->getStorage('commerce_product_variation')->load((int) $v);

      $variation_max_occupancy = preg_replace('/[^0-9]/', '', $product_variation->getTitle());

      $usable_variation = [];
      // @todo find a better way to know max occupancy for a given variation.
      if ($variation_max_occupancy == $data['values']['guests']) {
        $usable_variation['price'] = $product_variation->getPrice();
        $usable_variation['title'] = $product_variation->get('title')->get(0)->value;
        $usable_variation['variant_id'] = $product_variation->get('variation_id')->get(0)->value;
        break;
      }
    }

    $cart_manager  = \Drupal::service('commerce_cart.cart_manager');
    $cart_provider = \Drupal::service('commerce_cart.cart_provider');

    // @todo check "default" cart exists
    $cart = $cart_provider->getCart('default', $store);

    if (!$cart) {
      $cart = $cart_provider->createCart('default', $store);
    }
    else {
      // @todo allow more units per order
      $cart_manager->emptyCart($cart);
    }

    $order_item = \Drupal::entityTypeManager()->getStorage('commerce_order_item')->create([
      'title'            => $node->label(),
      'type'             => 'bee',
      'purchased_entity' => $usable_variation['variant_id'],
      'quantity'         => $data['norm']['dates_from_search_form']['days'],
      'unit_price'       => $usable_variation['price'],
    ]);

    $order_item->set('field_booking', $booking);
    $order_item->set('field_node', $node);
    $order_item->set('field_check_in', [
      $data['norm']['dates_from_search_form']['checkin']['Y-m-d-H-i-s'],
    ]);
    $order_item->set('field_check_out', [
      $data['norm']['dates_from_search_form']['checkout']['Y-m-d-H-i-s'],
    ]);

    $order_item->save();

    $cart_manager->addOrderItem($cart, $order_item);

    $session = \Drupal::request()->getSession();
    $session->set('beehotel_data', ['from' => $this->getFormId()]);

    $form_state->setRedirect('commerce_cart.page', ['commerce_order' => $cart->id()]);
  }

  /**
   * Load entity form display configuration.
   *
   * @param string $entity_type
   *   Entity type.
   * @param string $bundle
   *   Bundle of the entity.
   * @param string $mode
   *   Form mode.
   *
   * @return \Drupal\Core\Entity\EntityInterface|null
   *   EntityFormDisplay or null
   */
  public function getEntityFormConfiguration($entity_type, $bundle, $mode = 'default') {
    $config_keys = [$entity_type, $bundle, $mode];
    $config_id = implode('.', $config_keys);
    try {
      return $this->entityTypeManager->getStorage('entity_form_display')->load($config_id);
    }
    catch (PluginException $e) {
      return NULL;
    }
  }

}

==> bee_hotel-2.14.x-dev.tar.gz: bee_hotel/src/Form/BookThisUnitForm.php <==
<?php

namespace Drupal\bee_hotel\Form;

use Drupal\bee_hotel\Util\BeeHotelUnit;
use Drupal\bee_hotel\Util\Dates;
use Drupal\commerce_product\Entity\Product;
use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\node\NodeInterface;
use Drupal\node\Entity\Node;

define('FIELD_PRODUCT', 'field_product');

/**
 * Reservation form for a unit.
 *
 * @package Drupal\bee_hotel
 *   Defines the reservation form for a unit
 */
class BookThisUnitForm extends FormBase {

  /**
   * Constructs the object.
   */
  public function __construct() {
    $this->beehotelunit = new BeeHotelUnit($data = []);
    $this->beehoteldates = new Dates();
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'bee_hotel_book_this_unit_form';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state, NodeInterface $node = NULL) {

    $config = $this->config('beehotel.settings');

    if ($config->get("beehotel.off_value")) {
      \Drupal::messenger()->addWarning($config->get("beehotel.off_text"));
      return;
    }

    $data = [];

    $data['bee_settings'] = \Drupal::config('node.type.' . $node->bundle())->get('bee');

    $data['default_values']['dates'] = date("j M Y", strtotime("+1 day")) . " - " . date("j M Y", strtotime("+3 day"));

    $data['request_values']['row'] = \Drupal::request()->get('v');

    // Pre-fill for Litepicker.
    if (!empty($data['request_values']['row'])) {
      $data['request_values']['pieces'] = explode("-", $data['request_values']['row']);
      $this->beehoteldates->fromRequestToLitepicker($data);
    }

    $form['node'] = [
      '#type' => 'hidden',
      '#value' => $node->id(),
    ];

    $form['title'] = [
      '#type' => 'html_tag',
      '#tag' => 'h2',
      '#value' => $this
        ->t('Book @t now', ['@t' => $node->getTitle()]),
    ];

    $form['dates'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Check in > Check out'),
      '#default_value' => $data['default_values']['dates'],
      '#required' => TRUE,
    ];

    $product_field = $node->get(FIELD_PRODUCT)->getValue();
    $pid = $product_field[0]['target_id'];

    $data['max_occupancy'] = $this->beehotelunit->maxOccupancy($node);
    $options = [];
    for ($o = 1; $o <= $data['max_occupancy']; $o++) {
      $options[$o] = $o;
    }

    $form['guests'] = [
      '#type' => 'select',
      '#title' => $this->t('Guests'),
      '#options' => $options,
    ];

    if (isset($data['request_values']['pieces'][6])) {
      $form['guests']['#default_value'] = trim($data['request_values']['pieces'][6]);
    }

    $form['pid'] = [
      '#type' => 'hidden',
      '#value' => $pid,
    ];

    $form['submit'] = [
      '#type' => 'submit',
      '#value' => t('Search'),
    ];

    $form['#attached']['library'][] = 'bee_hotel/book-this-unit';
    $form['#weight'] = -200;

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {

    $data = [];
    $data['values'] = $form_state->getValues();
    $node = Node::load($data['values']['node']);
    $bee_settings = \Drupal::config('node.type.' . $node->bundle())->get('bee');
    $data = $this->beehotelunit->getAvailableUnits($data);

    if (empty($data['available_units'])) {
      $form_state->setError($form, t('Sorry, no Unit available now'));
    }

    // Payment validation.
    if ($bee_settings['payment'] < 1) {
      $form_state->setErrorByName('checkin', $this->t('No payment available for this Content type. Please check "Enable payment for bookings"'));
    }

    if ($node->get("field_accept_reservations")->value != 1) {
      $form_state->setErrorByName('checkin', $this->t('Sorry, no Unit available now...'));
    }

  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {

    $data = [];
    $data['values'] = $form_state->getValues();
    $node = Node::load($data['values']['node']);
    $bee_settings = \Drupal::config('node.type.' . $node->bundle())->get('bee');
    $data = $this->beehotelunit->getAvailableUnits($data);

    $booking = bat_booking_create([
      'type' => 'bee',
      'label' => $node->label(),
    ]);

    $booking->set('booking_start_date', $data['norm']['dates_from_search_form']['checkin']['Y-m-d-H-i-s']);
    $booking->set('booking_end_date', $data['norm']['dates_from_search_form']['checkout']['Y-m-d-H-i-s']);
    $booking->save();

    $product = $node->get(FIELD_PRODUCT)->entity;

    $stores = $product->getStores();
    $store = reset($stores);

    /*Load Product*/
    $product = Product::load((int) $data['values']['pid']);

    /*Load Product Variations*/
    $entity_manager = \Drupal::EntityTypeManager();
    $variations = $product->getVariationIds();

    $options = [];

    // This works fine for single slot units.
    foreach ($variations as $v) {
      $product_variation = $entity_manager->getStorage('commerce_product_variation')->load((int) $v);

      $variation_max_occupancy = preg_replace('/[^0-9]/', '', $product_variation->getTitle());

      $usable_variation = [];
      // @todo find a better way to know max occupancy for a given variation.
      if ($variation_max_occupancy == $data['values']['guests']) {
        $usable_variation['price'] = $product_variation->getPrice();
        $usable_variation['title'] = $product_variation->get('title')->get(0)->value;
        $usable_variation['variant_id'] = $product_variation->get('variation_id')->get(0)->value;
        break;
      }
    }

    $cart_manager  = \Drupal::service('commerce_cart.cart_manager');
    $cart_provider = \Drupal::service('commerce_cart.cart_provider');

    // @todo check "default" cart exists
    $cart = $cart_provider->getCart('default', $store);

    if (!$cart) {
      $cart = $cart_provider->createCart('default', $store);
    }
    else {
      // @todo allow more units per order
      $cart_manager->emptyCart($cart);
    }

    $order_item = \Drupal::entityTypeManager()->getStorage('commerce_order_item')->create([
      'title'            => $node->label(),
      'type'             => 'bee',
      'purchased_entity' => $usable_variation['variant_id'],
      'quantity'         => $data['norm']['dates_from_search_form']['days'],
      'unit_price'       => $usable_variation['price'],
    ]);

    $order_item->set('field_booking', $booking);
    $order_item->set('field_node', $node);
    $order_item->set('field_check_in', [
      $data['norm']['dates_from_search_form']['checkin']['Y-m-d-H-i-s'],
    ]);
    $order_item->set('field_check_out', [
      $data['norm']['dates_from_search_form']['checkout']['Y-m-d-H-i-s'],
    ]);

    $order_item->save();

    $cart_manager->addOrderItem($cart, $order_item);

    $session = \Drupal::request()->getSession();
    $session->set('beehotel_data', ['from' => $this->getFormId()]);

    $form_state->setRedirect('commerce_cart.page', ['commerce_order' => $cart->id()]);
  }

  /**
   * Load entity form display configuration.
   *
   * @param string $entity_type
   *   Entity type.
   * @param string $bundle
   *   Bundle of the entity.
   * @param string $mode
   *   Form mode.
   *
   * @return \Drupal\Core\Entity\EntityInterface|null
   *   EntityFormDisplay or null
   */
  public function getEntityFormConfiguration($entity_type, $bundle, $mode = 'default') {
    $config_keys = [$entity_type, $bundle, $mode];
    $config_id = implode('.', $config_keys);
    try {
      return $this->entityTypeManager->getStorage('entity_form_display')->load($config_id);
    }
    catch (PluginException $e) {
      return NULL;
    }
  }

}
